/**
 * DoubleDataRows.java
 * eu.gronos.kostenrechner.controller (Kostenrechner)
 */
package eu.gronos.kostenrechner.data.tenordaten;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXB;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Von {@link ArrayList} abgeleitete Klasse, die notwendig ist, weil sich in
 * {@link JAXB} keine mehrdimenensionalen Listen ohne eigene Klasse exportieren
 * lassen lassen.
 *
 * @author Peter Schuster (setrok)
 * @date 2 Jan 2019
 *
 */
@XmlRootElement(name = "zeile")
public class DoubleDataRows extends ArrayList<Double> {

	private static final long serialVersionUID = 1356128727610890073L;

	public DoubleDataRows() {
		super();
	}

	public DoubleDataRows(ArrayList<Double> dv) {
		super(dv.size());
		super.addAll(dv);
	}

	public DoubleDataRows(int size) {
		super(size);
	}

	@Override
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		for (Double d : this) {
			buffer.append(String.format("%,.2f ", d));
		}
		return buffer.toString();
	}

	/**
	 * Die Methode stellt die Zeile als Hintereinanderreihung von {@link Double}s
	 * dar.
	 * 
	 * @return alle Werte einer Zeile, als zweistellige {@link Double}s, mit
	 *         Leerzeichen getrennt
	 */
	@XmlAttribute(name = "spalten")
	public String getZeile() {
		return toString();
	}

	/**
	 * Die Methode trennt die Zeile, als zweistellige {@link Double}s, mit
	 * Leerzeichen getrennt, in eine {@link List} von {@link Double}s auf.
	 * 
	 * @param zeile alle Werte einer Zeile, als zweistellige {@link Double}s, mit
	 *              Leerzeichen getrennt, als {@link String}
	 */
	public void setZeile(String zeile) {
		if (zeile == null)
			return;
		for (String s : zeile.split("\\s+")) {
			if (s != null && !s.isEmpty()) {
				try {
					Double d = Double.parseDouble(s.replace(",", "."));
					this.add(d);
				} catch (NumberFormatException error) {
					System.out.println("Nix kann konvertieren: " + s);
				}
			}
		}
	}

}
